#####
##
# OrderedObjectManager / maik.jablonski@uni-bielefeld.de
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# This product uses modified codes from the OrderedFolder-Product
# written by Stephan Richter / copyrighted by iuveno AG
# Please read the LICENSE.txt!
##
####

from types import StringType
from Globals import ImageFile, DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager
from OFS.CopySupport import CopyContainer

oomPermission = 'Copy or Move'
ObjectManager.security = ClassSecurityInfo()

ObjectManager.manage_main = DTMLFile('dtml/manage_main', globals())

misc_ = {'up'     : ImageFile('www/up.gif', globals()),
         'down'   : ImageFile('www/down.gif', globals()),
         'top'    : ImageFile('www/top.gif', globals()),
         'bottom' : ImageFile('www/bottom.gif', globals())}
        

def get_object_position(self, id):
    """ Get the position of a particular object. """
    
    ids = map(lambda obj: obj['id'], self._objects)
    if id in ids:
        return ids.index(id)
    else:
        raise 'ObjectNotFound', self.aq_parent._objects        

ObjectManager.security.declareProtected(oomPermission, 'get_object_position')
ObjectManager.get_object_position=get_object_position    
    
    
def move_objects_by_positions(self, ids, position_delta):
    """ moves objects with ids by position_delta """

    if type(ids) is StringType: ids = (ids,)

    # Interestingly enough, we need to switch the order when
    # moving done, so that the movements won't cancel each other
    if position_delta > 0:
        ids = list(ids)
        ids.reverse()
            
    moved_objects = 0
        
    for id in ids:
        old_position = self.get_object_position(id)
        new_position = old_position + position_delta
        object = self._objects[old_position]
        # Make sure the new position makes sense and is valid
        if not (old_position == new_position  or
                new_position > len(self._objects) or
                new_position < 0):
            
            objects = list(self._objects)
            # now delete the entry ...
            objects = filter(lambda obj, id=id: obj['id'] != id, objects)
            # ... and now add it again
            objects.insert(new_position, object)
            self._objects = tuple(objects)

            moved_objects = moved_objects + 1

    return moved_objects

ObjectManager.security.declareProtected(oomPermission, 'move_objects_by_positions')
ObjectManager.move_objects_by_positions=move_objects_by_positions


def move_objects_up(self, ids):
    """ moving the items up """
        
    return self.move_objects_by_positions(ids, -1)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_up')
ObjectManager.move_objects_up=move_objects_up


def move_objects_down(self, ids):
    """ moving the items down """
 
    return self.move_objects_by_positions(ids, +1)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_down')
ObjectManager.move_objects_down=move_objects_down


def move_objects_to_top(self, ids):
    """ moving the items to the top """
               
    if type(ids) is StringType: ids = (ids,)

    position_delta = - self.get_object_position(ids[0])
    return self.move_objects_by_positions(ids, position_delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_top')
ObjectManager.move_objects_to_top=move_objects_to_top


def move_objects_to_bottom(self, ids):
    """ moving the items to the bottom """

    if type(ids) is StringType: ids = (ids,)

    # We will do the reverse twice,
    # but for going to the bottom there is no other choice.
    ids = list(ids)
    ids.reverse()

    position_delta = len(self.objectIds()) - self.get_object_position(ids[-1])
    return self.move_objects_by_positions(ids, position_delta)

ObjectManager.security.declareProtected(oomPermission, 'move_objects_to_bottom')
ObjectManager.move_objects_to_bottom=move_objects_to_bottom


def manage_move_objects_up(self, ids=None, REQUEST=None):
    """ moves selected items up by one """

    if type(ids) is StringType: ids = (ids,)
            
    attempt = 0
    if ids: attempt = self.move_objects_up(ids)

    if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
    return attempt

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_up')
ObjectManager.manage_move_objects_up=manage_move_objects_up


def manage_move_objects_down(self, ids=None, REQUEST=None):
    """ moves selected items down by one """

    if type(ids) is StringType: ids = (ids,)

    attempt = 0
    if ids: attempt = self.move_objects_down(ids)

    if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
    return attempt

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_down')
ObjectManager.manage_move_objects_down=manage_move_objects_down


def manage_move_objects_to_top(self, ids=None, REQUEST=None):
    """ moves selected items to top """
    
    if type(ids) is StringType: ids = (ids,)

    attempt=0
    if ids: attempt=self.move_objects_to_top(ids)

    if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
    return attempt

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_top')
ObjectManager.manage_move_objects_to_top=manage_move_objects_to_top


def manage_move_objects_to_bottom(self, ids=None, REQUEST=None):
    """ moves selected items to bottom """

    if type(ids) is StringType: ids = (ids,)

    attempt=0
    if ids: attempt = self.move_objects_to_bottom(ids)

    if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
    return attempt

ObjectManager.security.declareProtected(oomPermission, 'manage_move_objects_to_bottom')
ObjectManager.manage_move_objects_to_bottom=manage_move_objects_to_bottom


def manage_orderObjects(self, ids=None, REQUEST=None):
    """ orders a container with the ordered list 'ids' """

    if ids:
        for id in ids:
            old_position = self.get_object_position(id)
            new_position = ids.index(id)
            object = self._objects[old_position]
            # Make sure the new position makes sense and is valid
            if not (old_position == new_position  or
                    new_position > len(self._objects) or
                    new_position < 0):
                
                objects = list(self._objects)
                # now delete the entry ...
                objects = filter(lambda obj, id=id: obj['id'] != id, objects)
                # ... and now add it again
                objects.insert(new_position, object)
                self._objects = tuple(objects)

    if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        
ObjectManager.security.declareProtected(oomPermission, 'manage_orderObjects')
ObjectManager.manage_orderObjects = manage_orderObjects

# Here we patch the CopyContainer - 
# manage_renameObject should preserve the order

CopyContainer._old_manage_renameObject=CopyContainer.manage_renameObject

def manage_renameObject(self, id, new_id, REQUEST=None):
    """ rename a particular sub-object """
    
    position = self.get_object_position(id)
    result = self._old_manage_renameObject(id, new_id, REQUEST)
    newposition = self.get_object_position(new_id)
    self.move_objects_by_positions(new_id, position - newposition)
    return result

CopyContainer.manage_renameObject=manage_renameObject


InitializeClass(ObjectManager) 
